@@ -6,4 +6,5 @@ vivarium_cluster_tools
 Tools for working with :mod:`vivarium` on compute clusters.
 
 """
-from vivarium_cluster_tools.utilities import mkdir
+from vivarium_cluster_tools._version import __version__
+from vivarium_cluster_tools.utilities import get_cluster_name, mkdir
