@@ -7,6 +7,9 @@ Tools for interacting with the IHME cluster.
 
 """
 from vivarium_cluster_tools.psimulate.cluster.cli_options import (
+    MAX_RUNTIME_DEFAULT,
+    PEAK_MEMORY_DEFAULT,
+    with_hardware,
     with_peak_memory,
     with_project,
     with_queue_and_max_runtime,
