@@ -7,9 +7,41 @@ Command line options for configuring the cluster environment in psimulate runs.
 
 """
 
+from __future__ import annotations
+
 import click
 
+from vivarium_cluster_tools.cli_tools import CLIFunction
+
 _RUNTIME_FORMAT = "hh:mm:ss"
+MAX_RUNTIME_DEFAULT = "24:00:00"
+PEAK_MEMORY_DEFAULT = 3  # GB
+
+# https://docs.cluster.ihme.washington.edu/#hpc-execution-host-hardware-specifications
+_AVAILABLE_HARDWARE = [
+    "c6320",  # typical
+    "r630",  # high capacity
+    "c6420v1",  # batch 1
+    "c6420v2",  # batch 2
+    "r650",  # high capacity
+    "r650v2",  # high capacity
+    "r650xs",  # high speed
+]
+
+
+def _validate_and_split_hardware(
+    ctx: click.Context, param: click.core.Option, value: str | None
+) -> list[str]:
+    hardware: list[str] = value.split(",") if value else []
+    bad_requests = set(hardware) - set(_AVAILABLE_HARDWARE)
+    if bad_requests:
+        raise click.BadParameter(
+            f"Hardware request(s) {bad_requests} are not supported.\n"
+            f"Supported hardware requests: {_AVAILABLE_HARDWARE}.\n"
+            "Refer to https://docs.cluster.ihme.washington.edu/#hpc-execution-host-hardware-specifications"
+        )
+    return hardware
+
 
 with_project = click.option(
     "--project",
@@ -21,12 +53,12 @@ with_project = click.option(
             "proj_csu",
         ]
     ),
-    default="proj_simscience_prod",
+    required=True,
     help="The cluster project under which to run the simulation.",
 )
 
 
-def with_queue_and_max_runtime(func):
+def with_queue_and_max_runtime(func: CLIFunction) -> CLIFunction:
     """Provide a single decorator for both queue and max runtime
     since they are tightly coupled.
 
@@ -40,15 +72,32 @@ with_peak_memory = click.option(
     "--peak-memory",
     "-m",
     type=int,
-    default=3,
+    default=PEAK_MEMORY_DEFAULT,
+    show_default=True,
     help=(
-        "The estimated maximum memory usage in GB of an individual simulate job. "
-        "The simulations will be run with this as a limit."
+        "The memory request in GB of each individual simulation job. "
+        "The simulations will be killed if they exceed this limit."
     ),
 )
 
 
-def _queue_and_runtime_callback(ctx: click.Context, param: str, value: str) -> str:
+with_hardware = click.option(
+    "--hardware",
+    "-h",
+    help=(
+        "The (comma-separated) specific hardware to run the jobs on. This can be useful to request "
+        "specifically fast nodes ('-h r650xs') vs high capacity nodes ('-h r630,r650,r650v2'). "
+        "Note that the hardware changes on a roughly annual schedule. "
+        f"The currently-supported options are: {_AVAILABLE_HARDWARE}. "
+        "For details, refer to: https://docs.cluster.ihme.washington.edu/#hpc-execution-host-hardware-specifications"
+    ),
+    callback=_validate_and_split_hardware,
+)
+
+
+def _queue_and_runtime_callback(
+    ctx: click.Context, param: click.core.Parameter, value: str
+) -> str:
     if param.name == "queue" and "max_runtime" in ctx.params:
         runtime_string, queue = _validate_runtime_and_queue(ctx.params["max_runtime"], value)
         ctx.params["max_runtime"], value = runtime_string, queue
@@ -60,7 +109,7 @@ def _queue_and_runtime_callback(ctx: click.Context, param: str, value: str) -> s
     return value
 
 
-def _validate_runtime_and_queue(runtime_string: str, queue: str):
+def _validate_runtime_and_queue(runtime_string: str, queue: str | None) -> tuple[str, str]:
     try:
         hours, minutes, seconds = runtime_string.split(":")
     except ValueError:
@@ -89,15 +138,16 @@ def _validate_runtime_and_queue(runtime_string: str, queue: str):
     elif queue in max_runtime_map:
         # Things are peachy
         pass
-    else:
+    elif queue is None:
         # We need to set a default based on the runtime.
-        assert queue is None
         # First queue we qualify for.
         queue = [
             q
             for q, max_queue_runtime in max_runtime_map.items()
             if total_hours <= max_queue_runtime
         ][0]
+    else:
+        raise ValueError(f"Unexpected queue {queue} supplied.")
 
     return runtime_string, queue
 
@@ -117,13 +167,13 @@ _with_max_runtime = click.option(
     "--max-runtime",
     "-r",
     type=str,
-    default="24:00:00",
+    default=MAX_RUNTIME_DEFAULT,
+    show_default=True,
     help=(
-        f"The estimated maximum runtime ({_RUNTIME_FORMAT}) of the simulation jobs. "
-        "By default, the cluster will terminate jobs after 24h regardless of "
-        "queue. The maximum supported runtime is 3 days. Keep in mind that the "
+        f"The runtime request ({_RUNTIME_FORMAT}) of each individual simulation job. "
+        "The maximum supported runtime is 3 days. Keep in mind that the "
         "session you are launching from must be able to live at least as long "
-        "as the simulation jobs, and that runtimes by node vary wildly."
+        "as the simulation jobs and that runtimes by node vary wildly."
     ),
     callback=_queue_and_runtime_callback,
 )
