@@ -4,14 +4,17 @@ Cluster Interface
 =================
 
 """
+from __future__ import annotations
 
 import atexit
 import os
 import shutil
 from pathlib import Path
-from typing import Any, NamedTuple, TextIO
+from tempfile import _TemporaryFileWrapper
+from typing import NamedTuple
 
 from vivarium_cluster_tools.psimulate.environment import ENV_VARIABLES
+from vivarium_cluster_tools.utilities import get_drmaa
 
 
 def validate_cluster_environment() -> None:
@@ -27,30 +30,32 @@ class NativeSpecification(NamedTuple):
     job_name: str
     project: str
     queue: str
-    peak_memory: str
+    peak_memory: int  # Memory in GB
     max_runtime: str
+    hardware: list[str]
 
     # Class constant
     NUM_THREADS: int = 1
 
-    def to_cli_args(self):
+    def to_cli_args(self) -> str:
         return (
             f"-J {self.job_name} "
             f"-A {self.project} "
             f"-p {self.queue} "
             f"--mem={self.peak_memory*1024} "
             f"-t {self.max_runtime} "
-            f"-c {self.NUM_THREADS}"
-        )
+            f"-c {self.NUM_THREADS} "
+            f"{'-C ' + '|'.join(self.hardware) if self.hardware else ''}"
+        ).strip()
 
 
 def submit_worker_jobs(
     num_workers: int,
-    worker_launch_script: TextIO,
+    worker_launch_script: _TemporaryFileWrapper[str],
     cluster_logging_root: Path,
     native_specification: NativeSpecification,
 ) -> None:
-    drmaa = _get_drmaa()
+    drmaa = get_drmaa()
     s = drmaa.Session()
     s.initialize()
     jt = s.createJobTemplate()
@@ -83,15 +88,3 @@ def submit_worker_jobs(
                 raise
 
     atexit.register(kill_jobs)
-
-
-def _get_drmaa() -> Any:
-    try:
-        import drmaa
-    except (RuntimeError, OSError):
-        if "slurm" in ENV_VARIABLES.HOSTNAME.value:
-            ENV_VARIABLES.DRMAA_LIB_PATH.update("/opt/slurm-drmaa/lib/libdrmaa.so")
-            import drmaa
-        else:
-            drmaa = object()
-    return drmaa
