@@ -4,8 +4,10 @@ psimulate Jobs
 ==============
 
 """
+
+from collections import defaultdict
 from pathlib import Path
-from typing import List, NamedTuple, Tuple
+from typing import Any, NamedTuple
 
 import numpy as np
 import pandas as pd
@@ -18,30 +20,40 @@ class JobParameters(NamedTuple):
     """Parameters for a single distributed simulation job."""
 
     model_specification: str
-    branch_configuration: dict
+    branch_configuration: dict[str, Any]
     input_draw: int
     random_seed: int
     results_path: str
-    extras: dict
+    backup_configuration: dict[str, Any]
+    extras: dict[str, Any]
 
     @property
-    def shared(self) -> dict:
+    def shared(self) -> dict[str, Any]:
         """Parameters shared by all jobs in a psimulate run."""
         return {
             "model_specification": self.model_specification,
             "results_path": self.results_path,
+            "backup_configuration": self.backup_configuration,
         }
 
     @property
-    def job_specific(self) -> dict:
+    def job_specific(self) -> dict[str, Any]:
         """Parameters that vary by job in a psimulate run."""
         return {
+            **self.branch_configuration,
             "input_draw": self.input_draw,
             "random_seed": self.random_seed,
-            **self.branch_configuration,
         }
 
-    def to_dict(self) -> dict:
+    @property
+    def sim_config(self) -> dict[str, Any]:
+        """Parameters for the simulation configuration."""
+        config = defaultdict(dict, self.branch_configuration)
+        config["randomness"]["random_seed"] = self.random_seed
+        config["input_data"]["input_draw_number"] = self.input_draw
+        return dict(config)
+
+    def to_dict(self) -> dict[str, Any]:
         # I will never understand why this is a private
         # method of named tuples.
         return self._asdict()
@@ -58,24 +70,32 @@ def build_job_list(
     model_specification_path: Path,
     output_root: Path,
     keyspace: branches.Keyspace,
-    existing_outputs: pd.DataFrame,
-    extras: dict,
-) -> Tuple[List[dict], int]:
+    finished_sim_metadata: pd.DataFrame,
+    backup_freq: int | None,
+    backup_dir: Path,
+    backup_metadata_path: Path,
+    extras: dict[str, Any],
+) -> tuple[list[dict[str, str | int | dict[str, Any]]], int]:
     jobs = []
     number_already_completed = 0
 
     if command in [COMMANDS.run, COMMANDS.restart, COMMANDS.expand]:
-        for (input_draw, random_seed, branch_config) in keyspace:
+        for input_draw, random_seed, branch_config in keyspace:
             parameters = JobParameters(
                 model_specification=str(model_specification_path),
                 branch_configuration=branch_config,
                 input_draw=int(input_draw),
                 random_seed=int(random_seed),
                 results_path=str(output_root),
-                extras={},
+                backup_configuration={
+                    "backup_dir": backup_dir,
+                    "backup_freq": backup_freq,
+                    "backup_metadata_path": backup_metadata_path,
+                },
+                extras=extras,
             )
 
-            if already_complete(parameters, existing_outputs):
+            if already_complete(parameters, finished_sim_metadata):
                 number_already_completed += 1
             else:
                 jobs.append(parameters.to_dict())
@@ -88,30 +108,26 @@ def build_job_list(
                 input_draw=0,
                 random_seed=i,
                 results_path=str(output_root),
+                backup_configuration={},
                 extras={"test_type": extras["test_type"]},
             )
             jobs.append(parameters.to_dict())
 
-    np.random.shuffle(jobs)
+    np.random.shuffle(jobs)  # type: ignore [arg-type]
     return jobs, number_already_completed
 
 
-def already_complete(job_parameters: JobParameters, existing_outputs: pd.DataFrame) -> bool:
-    if existing_outputs.empty:
+def already_complete(
+    job_parameters: JobParameters, finished_sim_metadata: pd.DataFrame
+) -> bool:
+    if finished_sim_metadata.empty:
         return False
 
-    job_parameter_list = collapse_nested_dict(job_parameters.branch_configuration)
-    job_parameter_list.extend(
-        [
-            ("run_configuration.run_key.input_draw", job_parameters.input_draw),
-            ("run_configuration.run_key.random_seed", job_parameters.random_seed),
-        ]
-    )
-
-    mask = pd.Series(True, index=existing_outputs.index)
+    job_parameter_list = collapse_nested_dict(job_parameters.job_specific)
+    mask = pd.Series(True, index=finished_sim_metadata.index)
     for k, v in job_parameter_list:
         if isinstance(v, float):
-            mask &= np.isclose(existing_outputs[k], v)
+            mask &= np.isclose(finished_sim_metadata[k], v)
         else:
-            mask &= existing_outputs[k] == v
-    return np.any(mask)
+            mask &= finished_sim_metadata[k] == v
+    return bool(np.any(mask))
