@@ -4,24 +4,20 @@ Model specification management
 ==============================
 
 """
+
 import typing
 from pathlib import Path
-from typing import Optional
 
 import yaml
+from layered_config_tree import LayeredConfigTree
+from layered_config_tree.exceptions import ConfigurationError, ConfigurationKeyError
 from loguru import logger
-from vivarium.config_tree import ConfigurationKeyError
 from vivarium.framework.artifact import parse_artifact_path_config
-from vivarium.framework.configuration import (
-    ConfigTree,
-    ConfigurationError,
-    build_model_specification,
-)
+from vivarium.framework.configuration import build_model_specification
 
 from vivarium_cluster_tools.psimulate import COMMANDS
 
 if typing.TYPE_CHECKING:
-    # Cyclic import.
     from vivarium_cluster_tools.psimulate.branches import Keyspace
 
 
@@ -35,12 +31,12 @@ RESULTS_DIRECTORY_KEY = "results_directory"
 
 def parse(
     command: str,
-    input_model_specification_path: Optional[Path],
-    artifact_path: Optional[Path],
+    input_model_specification_path: Path | None,
+    artifact_path: Path | None,
     model_specification_path: Path,
     results_root: Path,
     keyspace: "Keyspace",
-) -> ConfigTree:
+) -> LayeredConfigTree:
     if command in [COMMANDS.restart, COMMANDS.expand]:
         return build_model_specification(model_specification_path)
     if command == COMMANDS.load_test:
@@ -82,7 +78,9 @@ def parse(
 
 
 def persist(
-    model_specification: ConfigTree,
+    model_specification: LayeredConfigTree,
     model_specification_path: Path,
 ) -> None:
-    model_specification_path.write_text(yaml.dump(model_specification.to_dict()))
+    model_specification_path.write_text(
+        yaml.dump(model_specification.to_dict(), sort_keys=False)
+    )
