@@ -9,8 +9,8 @@ Tools to manage and validate reproducible Python environments for simulation run
 import os
 import subprocess
 from pathlib import Path
-from typing import Dict, List, Tuple
 
+import click
 from loguru import logger
 
 
@@ -39,7 +39,7 @@ def validate(environment_file: Path) -> None:
         )
 
 
-def _parse_package_version(s: str) -> Tuple[str, str]:
+def _parse_package_version(s: str) -> tuple[str, str]:
     if (
         "no version control" in s
     ):  # installed from non-git controlled source code in editable mode
@@ -56,12 +56,12 @@ def _parse_package_version(s: str) -> Tuple[str, str]:
     return package, version
 
 
-def _convert_pip_list_to_dict(pf_list: List[str]) -> Dict:
+def _convert_pip_list_to_dict(pf_list: list[str]) -> dict[str, str]:
     return {p: v for p, v in [_parse_package_version(s) for s in pf_list]}
 
 
-def _compare_environments(current: Dict, original: Dict) -> None:
-    differences = []
+def _compare_environments(current: dict[str, str], original: dict[str, str]) -> None:
+    differences: list[str] = []
 
     current_packages = set(current.keys())
     original_packages = set(original.keys())
@@ -80,7 +80,7 @@ def _compare_environments(current: Dict, original: Dict) -> None:
             f"environment: {missing_packages}."
         )
 
-    differing_versions = []
+    differing_versions: list[str] = []
     for p in current_packages.intersection(original_packages):
         if current[p] != original[p]:
             differing_versions.append(f"{p}: {original[p]} -> {current[p]}")
@@ -92,9 +92,12 @@ def _compare_environments(current: Dict, original: Dict) -> None:
         )
 
     if differences:
-        differences = "\n".join(differences)
-        raise ValueError(
-            f"Differences found between environment used for original run and current environment. "
-            f"In order to successfully run, you should make a new environment using the requirements.txt "
-            f"file found in the output directory. Differences found as follows: {differences}."
+        differences_str = "\n".join(differences)
+        logger.info(
+            "Differences found between current environment and original environment used for "
+            f" this run. \n\nDifferences found are as follows: {differences_str}. Would you like to proceed?"
+        )
+        click.confirm(
+            "Would you like to proceed psimulate restart with the new environment?",
+            abort=True,
         )
