@@ -6,7 +6,10 @@ redis_dbs
 Redis database management.
 
 """
-from vivarium_cluster_tools.psimulate.redis_dbs.cli_options import with_redis
+from vivarium_cluster_tools.psimulate.redis_dbs.cli_options import (
+    with_max_workers,
+    with_redis,
+)
 from vivarium_cluster_tools.psimulate.redis_dbs.launcher import (
     launch_redis_processes as launch,
 )
