@@ -6,16 +6,43 @@ vivarium_cluster_tools Utilities
 Making directories is hard.
 
 """
+
 import functools
 import os
+import socket
 import time
 import warnings
+from collections.abc import Callable
 from pathlib import Path
-from typing import Callable, Union
+from typing import Any, ParamSpec, TypeVar
+
+from vivarium_cluster_tools.psimulate.environment import ENV_VARIABLES
+
+NUM_ROWS_PER_CENTRAL_LOG_FILE = 100_000
+
+P = ParamSpec("P")
+T = TypeVar("T")
+
+
+def get_cluster_name() -> str:
+    """Returns the hostname"""
+    return socket.gethostname()
+
+
+def get_drmaa() -> Any:
+    try:
+        import drmaa
+    except (RuntimeError, OSError):
+        if "slurm" in ENV_VARIABLES.HOSTNAME.value:
+            ENV_VARIABLES.DRMAA_LIB_PATH.update("/opt/slurm-drmaa/lib/libdrmaa.so")
+            import drmaa
+        else:
+            drmaa = object()
+    return drmaa
 
 
 def mkdir(
-    path: Union[str, Path], umask: int = 0o002, exists_ok: bool = False, parents: bool = False
+    path: str | Path, umask: int = 0o002, exists_ok: bool = False, parents: bool = False
 ) -> None:
     """Utility method to create a directory with specified permissions.
 
@@ -29,7 +56,6 @@ def mkdir(
         If False, raises FileExistsError if the directory already exists.
     parents
         If False, raises FileNotFoundError if the directory's parent doesn't exist.
-
     """
     path = Path(path)
     old_umask = os.umask(umask)
@@ -40,10 +66,10 @@ def mkdir(
 
 
 def backoff_and_retry(
-    backoff_seconds: Union[int, float] = 30,
+    backoff_seconds: int | float = 30,
     num_retries: int = 3,
     log_function: Callable[[str], None] = warnings.warn,
-):
+) -> Callable[[Callable[P, T]], Callable[P, T]]:
     """Adds a retry handler to the decorated function.
 
     Parameters
@@ -56,11 +82,14 @@ def backoff_and_retry(
         A callable used to emit retry messages.  Defaults to emitting
         a standard library warning.
 
+    Returns
+    -------
+        A function that retries the decorated function a specified number of times.
     """
 
-    def _wrap(func):
+    def _wrap(func: Callable[P, T]) -> Callable[P, T]:
         @functools.wraps(func)
-        def _wrapped(*args, **kwargs):
+        def _wrapped(*args: P.args, **kwargs: P.kwargs) -> T:
             retries = num_retries
             while retries:
                 try:
@@ -68,7 +97,7 @@ def backoff_and_retry(
                     break
                 except Exception as e:
                     log_function(
-                        f"Error trying to write results to hdf, retries remaining {retries}"
+                        f"Error trying to write results, retries remaining {retries}"
                     )
                     time.sleep(backoff_seconds)
                     retries -= 1
