@@ -6,37 +6,25 @@ Performance Reporting
 Tools for summarizing and reporting performance information.
 
 """
+
 import json
 import re
 from pathlib import Path
-from typing import Tuple, Union
+from typing import Generator
 
 import numpy as np
 import pandas as pd
 import requests
 from loguru import logger
-from pandas import json_normalize
 
 BASE_PERF_INDEX_COLS = ["host", "job_number", "task_number", "draw", "seed"]
 
 # The number of scenario columns beyond which we shorten the scenarios to a single string
 COMPOUND_SCENARIO_COL_COUNT = 2
 
-# Scenario columns that are not useful describe a scenario or are duplicated
-EXTRANEOUS_SCENARIO_COLS = [
-    "scenario_run_configuration_run_id",
-    "scenario_run_configuration_results_directory",
-    "scenario_run_configuration_run_key_input_draw",
-    "scenario_run_configuration_run_key_random_seed",
-    "scenario_randomness_random_seed",
-    "scenario_randomness_additional_seed",
-    "scenario_input_data_input_draw_number",
-]
-
 
 class PerformanceSummary:
-    """
-    A class to implement a getter for data in the workers' performance logs.
+    """A class to implement a getter for data in the workers' performance logs.
 
     Given a Path, a PerformanceSummary class provides a generator to get at each
     entry in the workers' performance logs. The class also provides a method
@@ -47,6 +35,8 @@ class PerformanceSummary:
     ----------
     log_dir
         Path of log_dir
+    errors
+        Number of errors encountered while parsing logs
 
     """
 
@@ -54,7 +44,7 @@ class PerformanceSummary:
         self.log_dir: Path = log_dir
         self.errors: int = 0
 
-    def get_summaries(self) -> dict:
+    def get_summaries(self) -> Generator[pd.DataFrame, None, None]:
         """Generator to get all performance summary log messages in PerformanceSummary"""
         for log in [
             f for f in self.log_dir.iterdir() if self.PERF_LOG_PATTERN.fullmatch(f.name)
@@ -73,10 +63,10 @@ class PerformanceSummary:
                         continue
                     m = self.TELEMETRY_PATTERN.fullmatch(str(message))
                     if m:
-                        yield json_normalize(json.loads(message), sep="_")
+                        yield pd.json_normalize(json.loads(message), sep="_")
 
     def to_df(self) -> pd.DataFrame:
-        perf_data = []
+        perf_data: list[pd.DataFrame] = []
         for item in self.get_summaries():
             perf_data.append(item)
         if len(perf_data) < 1:
@@ -94,19 +84,27 @@ class PerformanceSummary:
     TELEMETRY_PATTERN = re.compile(r"^{\"host\".+\"job_number\".+}$")
     PERF_LOG_PATTERN = re.compile(r"^perf\.([0-9]+)\.([0-9]+)\.log$")
 
+    def clean_perf_logs(self) -> None:
+        """Remove all performance logs from the log_dir (after to_df has been called)"""
+        for log in [
+            f for f in self.log_dir.iterdir() if self.PERF_LOG_PATTERN.fullmatch(f.name)
+        ]:
+            log.unlink()
 
-def set_index_scenario_cols(perf_df: pd.DataFrame) -> Tuple[pd.DataFrame, list]:
+
+def set_index_scenario_cols(perf_df: pd.DataFrame) -> tuple[pd.DataFrame, list[str]]:
     """Get the columns useful to index performance data by."""
     index_cols = BASE_PERF_INDEX_COLS
-    perf_df = perf_df.drop(EXTRANEOUS_SCENARIO_COLS, axis=1)
     scenario_cols = [col for col in perf_df.columns if col.startswith("scenario_")]
     index_cols.extend(scenario_cols)
     perf_df = perf_df.set_index(index_cols)
     return perf_df, scenario_cols
 
 
-def add_squid_api_data(perf_df: pd.DataFrame):
-    """Given a dataframe from PerformanceSummary.to_df, add Squid API data for the job.
+def add_squid_api_data(perf_df: pd.DataFrame) -> pd.DataFrame:
+    """Add Squid API data to the performance dataframe.
+
+    Given a dataframe from PerformanceSummary.to_df, add Squid API data for the job.
     Squid API reference: https://hub.ihme.washington.edu/display/SCKB/How+to+use+Squid+API
     """
     try:
@@ -123,12 +121,15 @@ def add_squid_api_data(perf_df: pd.DataFrame):
             right_on=["squid_api_job_id"],
         )
     except Exception as e:
-        print(f"Squid API request failed with: {e}")
+        logger.warning(f"Squid API request failed with: {e}")
     return perf_df
 
 
-def print_stat_report(perf_df: pd.DataFrame, scenario_cols: list):
-    """Print some helpful stats from the performance data, grouped by scenario_cols"""
+def print_stat_report(perf_df: pd.DataFrame, scenario_cols: list[str]) -> None:
+    """Print some helpful stats from the performance data.
+
+    The stats are grouped by scenario_cols.
+    """
     pd.set_option("display.max_rows", None)
     pd.set_option("display.max_columns", None)
     pd.options.display.float_format = "{:.2f}".format
@@ -143,7 +144,7 @@ def print_stat_report(perf_df: pd.DataFrame, scenario_cols: list):
         )
         perf_df["compound_scenario"] = (
             perf_df[scenario_cols]
-            .to_csv(header=None, index=False, sep="/")
+            .to_csv(header=False, index=False, sep="/")
             .strip("\n")
             .split("\n")
         )
@@ -183,19 +184,23 @@ def print_stat_report(perf_df: pd.DataFrame, scenario_cols: list):
 
 
 def report_performance(
-    input_directory: Union[Path, str],
-    output_directory: Union[Path, str],
+    input_directory: Path | str,
+    output_directory: Path | str,
     output_hdf: bool,
     verbose: int,
-):
-    """Main method for vipin reporting. Gets job performance data, outputs to a file, and logs a report."""
+) -> pd.DataFrame | None:
+    """Main method for vipin reporting.
+
+    Gets job performance data, outputs to a file, and logs a report.
+    """
     input_directory, output_directory = Path(input_directory), Path(output_directory)
     perf_summary = PerformanceSummary(input_directory)
 
     perf_df = perf_summary.to_df()
+
     if len(perf_df) < 1:
         logger.warning(f"No performance data found in {input_directory}.")
-        return  # nothing left to do
+        return None  # nothing left to do
 
     # Add jobapi data about the job to dataframe
     perf_df = add_squid_api_data(perf_df)
@@ -212,6 +217,9 @@ def report_performance(
         out_file = out_file.with_suffix(".csv")
         perf_df.to_csv(out_file)
 
+    # Clean up performance logs
+    perf_summary.clean_perf_logs()
+
     if verbose:
         print_stat_report(perf_df, scenario_cols)
 
@@ -223,4 +231,5 @@ def report_performance(
         f'Performance summary {"hdf" if output_hdf else "csv"} can be found at {out_file}, with '
         f'{perf_df.shape[0]} row{"s" if perf_df.shape[0] > 1 else ""}.'
     )
-    return
+
+    return perf_df
